# This example solves a finite elasticity deformation of a wedge 
# loaded by pressure on one face. The symmetry of the problem 
# likens itself to a square-cornered cylinder with a pressure bc 
# on the 'inside' face
#    
#      y
#      ^
#      |
#    2 |\
#      | \
#      |  \     with unit depth
#    1 |\  \
#      | \  \
#      |  \  \
#      --------------> x  
#         1  2
#
# where the x-face (1,0) -> (2,0) was constrained to slide along the
# x-axis, the y-face (0,1) -> (0,2) was constrained to slide 
# along y and one z-face is completely fixed


#
# Set up local paths
#

$OUTPUT_DIR = "output" # local directory for problem output


#
# Set up problem
#
fem
fem define parameters;r;pressure_inflation;example
fem define coordinates;r;3d_Standard_RC;example reg 1
fem define bases;r;3Linear_3x3x3Gauss;example reg 1          # Geometry
fem define ;add bases;r;2Linear_3x3Gauss;example reg 1       # Faces
fem define ;add bases;r;5Auxiliary_3x3x3Gauss;example reg 1  # Pressure
fem define nodes;r;wedge;example reg 1 
fem define elements;r;wedge;example reg 1
fem define fibres;r;wedge;example reg 1
fem define elements;r;wedge;example fibre reg 1
fem define equation;r;Incompressible_Finite_Elasticity;example reg 1
fem define material;r;Orthotropic_Incompressible;example reg 1
fem define active;r;active0_00;example reg 1
fem define initial;r;pressure_inflation;example reg 1
fem define solve;r;newton;example reg 1

#
# Inital geometry
#
fem export nodes;wedge;$OUTPUT_DIR as wedge
fem export elem;wedge;$OUTPUT_DIR as wedge

#
#Solve 
#
set out;steps_inflate on

$NAME1 = "wedge_def_00"
$NAME2 = "_active_0000"

$ITERS = 5
$TOL = 0.0001

#
#increase pressure incrementally
#
for $PRESS ( 0..5 ) 
{

  if( $PRESS < 1 )
  {
    $FILENAME = "${NAME1}${PRESS}$NAME2"
    $INCREM = 0.0
  } else {
    $FILENAME = "${NAME1}${PRESS}$NAME2"
    $INCREM = 0.2
  }

  list assign 
  fem solve increment $INCREM iter $ITERS error $TOL
  fem export nodes;$FILENAME;$OUTPUT_DIR field as wedge
  fem export elements;$FILENAME;$OUTPUT_DIR field as wedge
  fem define initial;w;$FILENAME;$OUTPUT_DIR

}


set out off

fem define window
fem draw lines
fem draw lines deformed dotted 

