#
# A CMISS command file containing the AndreSolve subroutine. This is a 
# subroutine which is used in all the distributed cell based modelling that I
# do - easier to have it stored in one place than to have to modify it in each
# simulation command file.
#
# AndreSolve is a routine to effectively perform the Newton iterations required
# to obtain a converged solution - for a specific point in time (or, 
# equivalently, for a specific [Ca]i) iterate until the tension generated by 
# the cell model is at equilibrium with the passive stress applied by the 
# tissue.

# Variables which must be defined:
#
#
#

# This number is used for cool stuff
$BIG_NUMBER = 1.0e6;

sub AndreSolve {
    # Parameters
    ($current_time,$previous_time,$matrix_file,$g_element_name) = @_;

    $dt = $current_time - $previous_time;
    $local_time = $previous_time;

    print "\n$Fblack$Bwhite  Entering AndreSolve                                                                     $FBnone\n";
    print $Fgreen."Attempting to solve from $previous_time to $current_time$FBnone\n\n";
    
    # The minimum time step allowed
    $MIN_DT = 0.01;
    # The maximum number of iterations to take before 
    # altering the time step for the cell solution
    $MAX_ITERATIONS_DT = 20;
    # The maximum number of iterations permitted, exits
    # when this number is reached, regardless of whether
    # convergence has been reached or not
    $MAX_ITERATIONS = ($dt/$MIN_DT)*$MAX_ITERATIONS_DT;
    #$MAX_ITERATIONS = $MAX_ITERATIONS_DT;
    # The local total iterations counter
    $iterations = 0;
    # The local iterations counter for each time step
    $iterations_dt = 0;
    # Keep track of restarts
    $MAX_RESTARTS = 1;
    $number_of_restarts = 0;
    # This is the ratio used for convergence tests in the backend,
    # which we will use to determine whether or not to update the
    # cellular solution
    $CONVERGENCE_RATIO = $BIG_NUMBER;
    # Used to check the maximum absolute extension ratio difference
    # over all grid points between iterations
    $L_MAXABSDIFF = $BIG_NUMBER;
    # The tolerance for solving the cell model - i.e. the cell model is
    # only solve if the cellular extension ratio changes by more than this
    $L_TOLERANCE = 0.05;
    # The scale factor used to set the amount of cellular active tension
    # applied to the distributed mechanics
    $T_SCALE_FACTOR = 0.9;
    # Flag if the cell model is solved
    $CELL_SOLVED = 0;
    
    # The solution (Newton) iteration loop
    # We want to loop until either the final time is
    # reached or the maximum number of iterations is
    # exceeded
    set echo;
    while ($iterations < $MAX_ITERATIONS) {
        $iterations++;
        $iterations_dt++;
        $history_time++;
        print "$Fyellow$Bred  Iteration: $iterations ($history_time)  $FBnone\n";
        # Only solve the cellular model is the forces are not
        # yet in equilibrium
        $local_tend = $local_time + $dt;
        $CELL_SOLVED = 0;
        if ($CONVERGENCE_RATIO > $MECH_ERRTOL) {
            # ??????
            # Update the grid points - does this belong here or should the
            # grid points be updated within the Newton iterations ??
            # ??????
            fem update grid geometry deformed class $CELL from_class $MECHANICS;
            fem update grid metric deformed class $CELL from_class $MECHANICS;
            fem update grid material class $CELL;
            # Compute and store fibre extension ratio for
            # the cell model
            if ($SOLVE_DISTRIBUTED_MECHANICS) {
                fem update grid extension_ratio component $L_MECHIDX $L_GRIDARRAY $L_GRIDIDX class $MECHANICS;
            }
            # Check the change in extension ratio
            if ($iterations_dt > 1) {
                fem compare gridvars $L_GRIDARRAY field1 $L_GRIDIDX field2 $L_GRIDIDX_PREV return maxabsdiff to_variable L_MAXABSDIFF;
                print $Fyellow."L_MAXABSDIFF = $L_MAXABSDIFF$FBnone\n";
            }
            # Only solve the cell model if there is a significant
            # change in extension ratio values to try and stop numerical
            # instabilities
            if ($L_MAXABSDIFF > $L_TOLERANCE) {
                if ($SOLVE_CELL_MODEL) {
                    # Solve the cellular model 
                    fem solve class $CELL from $local_time to $local_tend;
                    $CELL_SOLVED = 1;
                } else {
                    $CELL_SOLVED = 0;
                }
            }
        } # if ($CONVERGENCE_RATIO > $MECH_ERRTOL)
        # Write to the history file
        fem write history unit $ITERATIONS_UNIT time $history_time variables yqs $FORMAT class $CELL;
        if ($CELL_SOLVED) {
            # Interpolate active tension from grid point variables 
            # to Gauss pts - checking if we want to exclude an 
            # element group
            if ((defined $exlude_element_group_named_INFARCT) && $exlude_element_group_named_INFARCT) {
                print $Fgreen."Excluding element group named INFARCT$FBnone\n";
                fem update gauss gridvars $T_GRIDARRAY $T_GRIDIDX $T_GAUSSARRAY $T_FROM_CELLS_GAUSSIDX scale_factor $T_SCALE_FACTOR exclude element INFARCT;
            } else {
                fem update gauss gridvars $T_GRIDARRAY $T_GRIDIDX $T_GAUSSARRAY $T_FROM_CELLS_GAUSSIDX scale_factor $T_SCALE_FACTOR;
            }
        }
        # Copy active tension into place for the distributed mechanics to 
        # use
        fem update gauss gauss_field destination_field $T_GAUSSIDX source_field $T_FROM_CELLS_GAUSSIDX;
        # Solve the distributed mechanics
        if ($SOLVE_DISTRIBUTED_MECHANICS) {
            fem solve class $MECHANICS increment 0.0 iterate 1 error $MECH_ERRTOL;
        } else {
            $CONVERGED = 1;
        }
        if ($output_iterations) {
            if ($export_potentials) {
                fem export element;$output/iter_potential_$history_time field as $g_element_name class $CELL;
            }
            if ($export_gauss) {
                # Export the strains
                fem update gauss strain fibre components;
                fem export gauss;$output/iter_gauss_strain_$history_time yg as $g_element_name;
                # and active stress
                fem update gauss gauss_field destination_field $T_GAUSSIDX source_field $T_FROM_CELLS_GAUSSIDX;
                fem update gauss stress fibre active components nominal;
                fem export gauss;$output/iter_gauss_active_stress_$history_time yg as $g_element_name;
                fem update gauss gauss_field destination_field $T_GAUSSIDX source_field $T_FROM_CELLS_GAUSSIDX;
            }
            if ($list_strains) {
                # List out the strain at the specified Gauss points
                $gauss_string = "";
                foreach $gauss_point (@gauss_points) {
                    $strain_file_name = $output."/gauss_point_".$gauss_point."_strain_".$history_time;
                    fem list strain;$strain_file_name gauss at $gauss_point fibre;
                    $gauss_string .= "$gauss_point,";
                }
                chop $gauss_string;
                # and the grid point closest to them
                fem evaluate strain;$output/grid_point_strains_iterations_$history_time from cell_extension_ratios index $L_GRIDIDX at gauss $gauss_string nearest class $CELL;
            }
            # and geometry
            fem export node;$output/deformed_iter_$history_time field as $g_element_name;
        }
        if ($CONVERGED) {
            # A converged solution was obtained, so we are done
            # for this time step..
            # Store the current cell state for the next set of 
            # iterations
            fem write matrix;$matrix_file matrices YQ,YQS $FORMAT;
            # and update the saved converged mechanics solution
            fem update solution converged_reference save class $MECHANICS;
            # Write to the log file
            print LOG "Converged solution found\n";
            print LOG "  Local time: $local_tend\n";
            print LOG "  Number of iterations: $iterations_dt\n";
            # and exit the loop if we have finished time
            if ($local_tend >= $current_time) {
                # We have integrated through time as far as we
                # need to go, so exit the loop
                last;
            } else {
                # Have not reached the end of the integration
                # interval, so continue interating
                $local_time += $dt;
                $iterations_dt = 0;
                # Need to make sure that the cell model gets solved
                $CONVERGENCE_RATIO = $BIG_NUMBER;
                $L_MAXABSDIFF = $BIG_NUMBER;
            }
        } else {
            # The solution has not yet converged, so iterate again
            # Check if it is time to change the time step size
            if (($CONVERGENCE_RATIO > $BIG_NUMBER) || ($iterations_dt >= $MAX_ITERATIONS_DT)) {
                if ($CONVERGENCE_RATIO > $BIG_NUMBER) {
                    print $Fgreen."CONVERGENCE_RATIO limit exceeded, decreasing DT$FBnone\n";
                } else {
                    print $Fgreen."MAX_ITERATIONS_DT reached, decreasing DT$FBnone\n";
                }
                # Make sure that the cell model is solved
                $CONVERGENCE_RATIO = $BIG_NUMBER;
                # and reset the cell matrices to the intial state
                fem read matrix;$matrix_file matrices YQ,YQS $FORMAT;
                $iterations_dt = 0;
                if ($dt/2 > $MIN_DT) {
                    # Half the time step
                    $dt /= 2.0;
                    print $Fgreen."Halfing time step - DT = $dt$FBnone\n";
                    # Need to reset the distributed solution
                    fem update solution converged_reference restore class $MECHANICS;
                } elsif ($dt > $MIN_DT) {
                    $dt = $MIN_DT;
                    print $Fgreen."DT hit minimum - DT = $dt$FBnone\n";
                    # Need to reset the distributed solution
                    fem update solution converged_reference restore class $MECHANICS;
                } else {
                    # DT is already minimal, so do not alter
                    print $Fgreen."DT is already minmal - DT = $dt$FBnone\n";
                    # try again, but only a few times
                    if ($number_of_restarts > $MAX_RESTARTS) {
                        print $Fyellow."Too many restarts ($number_of_restarts)...$FBnone\n";
                        last;
                    } else {
                        $number_of_restarts++;
                        # Need to reset the distributed solution
                        fem update solution converged_reference restore class $MECHANICS;
                    }
                }
            }
        }
    } # while ($iterations < $MAX_ITERATIONS)
    set echo;

    print "\n$Fwhite$Bblack  Leaving AndreSolve                                                                      $FBnone\n";
    # Return the state of convegence and number of 
    # iterations
    return ($CONVERGED,$iterations);
} # sub AndreSolve
