# Example showing how to use a cellml model

# We want to run the same model for two different parameter sets, so
# set up a function to do the solve and call it for each parameter set
sub runModel {
    ($suffix) = @_;

    # Define the CellML model to use and map the parameters to CMISS and
    # specify initial/boundary conditions for the cell model.
    fem define cell;r;$example/hill_$suffix;
    # Initialise the spatial properties of the model (there are none).
    fem define material;r;$example/hill cell;
    fem define material;r;$example/hill;
    fem update grid material;

    # Define the zero transmembrane flux boundary condition.
    fem define initial;r;$example/hill;

    # Define an explicit Adams integrator.
    fem define solve;r;$example/hill;

    # Initialise the solve.
    fem solve to 0;

    # Create a history file to store output variables of interest.
    fem open history;hill write variables yqs niqslist 1..3 binary;
    fem write history time 0 variables yqs binary;

    # Loop through time integrating the cellular model.
    $Tend = 1000;
    $dt = 10.0;
    for ($time=$dt;$time<$Tend;$time+=$dt) {
        fem solve restart to $time;
        fem write history time $time variables yqs binary;
    }
    fem close history binary;

    # Evaluate signals for the variables from the history file.
    fem evaluate electrode;hill_$suffix history hill from grid yqs iy 1..3 binary;

    # Define an export of the CMISS signals to UnEMAP signals..
    fem define export;r;$example/hill;

    # ..and export the signals to a UnEMAP signal file.
    fem export signal;hill_$suffix electrodes signal hill_$suffix names cell;
    
} # sub runModel


# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

fem define parameters;r;$example/hill;

# default nodes, elements and grids to get a single grid point
fem define node;d;
fem define bas;d;
fem define elem;d;
fem define grid;d;
fem update grid geometry;
fem update grid metric;

# Define the a user defined CellML electrophysiological model
fem define equation;r;$example/cellml;

# Run the model for 0 degrees C
&runModel("0degC");
# Run the model for 23 degrees C
&runModel("23degC");

# For testing we compare the generated CMISS signal files
if ($TESTING) {
    fem compare signal;hill_0degC basis both masterfile $example/hill_0degC comparefile hill_0degC binary;
    fem compare signal;hill_23degC basis both masterfile $example/hill_23degC comparefile hill_23degC binary;
}
