if( !$example ) {
  $example = ".";
}

# This example is designed to go through the whole process of getting current lines
# within a boundary element torso using streamlines. This is achieved by putting the torso
# inside a box and then defining a grid over the box. A domain solution is then performed
# at each grid point to get an element based representation of the potential gradient fields.
# A Perl routine is then used to combine the x, y and z components of the gradient field into
# a single exelem file.

# Set up the three regions for the example
$Torso = 1;
$Heart = 2;
$Box = 3;

# Set up the two classes for the example
$Dipole = 1;
$Grid = 2;

# Define the problem setup - 3d with 3 regions
fem define para;r;$example/torsotimedep 
fem define regi;r;$example/torsouno     
fem define coor 3,1

# Read in the geometry of the 3 regions
fem define node;r;$example/torsotimedep region $Torso,$Heart  
fem define node;r;$example/volume reg $Box
fem define base;r;$example/volume                 
fem define elem;r;$example/torsotimedep region $Torso,$Heart 
fem define elem;r;$example/volume reg $Box

# Define a finite difference grid over the box that encloses the
# torso. The potential field will be evaluated at these points.
fem define grid;r;$example/45k reg $Box
fem update grid geom reg $Box

# Define a quasistatic Laplace equation in the torso regions
# along with the material parameters and initial conditions etc.
fem define equa;r;$example/torsotimedep region $Torso,$Heart class $Dipole lock
fem define mate;r;$example/torsotimedep region $Torso,$Heart
fem define init;r;$example/torsotimedep region $Torso,$Heart
fem define sour;r;$example/test region $Heart 
fem define coup;r;$example/torsotimedep
fem define solv;r;$example/torsotimedep coupled region $Torso,$Heart

# A grid equation needs to be defined so we have a grid class in
# which to store the potentials. Here a Laplace equation is used for
# this purpose.
fem define equa;r;$example/volume cl $Grid reg $Box

# Solve the torso problem. In this case it has been set up to solve for
# only the time step that we want to export the potentials from.
fem solve coupled region $Torso,$Heart class $Dipole

# Export the geometry of the torso and epicardial surfaces
fem export node;hearttorso as hearttorso
fem export elem;hearttorso as hearttorso

# Export the potential field over the torso and epicardial surfaces
fem export node;field as hearttorso field
fem export elem;field as hearttorso field

# Perform a boundary element domain solution at each of the grid points
# to find the value of the potential gradient fields. There are two easy ways to get
# better, smoother streamlines but both take substantially longer.
# They involve:
#   Increasing grid density
#   Using adaptive integration 
# This evaluation provides linear speedup with accurate answers when 
# mulitprocessing (if not it certainly has done in the past).
fem evaluate solution grid element 345 optimise iy 1 gradient

# Export the geometry of the box that encloses the torso
fem export node;geom as test reg $Box
fem export elem;geom as test reg $Box

# Export the element based fields that contains the potential gradient fields evaluated
# at all grid points in the torso. All points outside the torso have been set to
# 0.0mV based of the C(P) coefficient that is calculated by the boundary element
# method.
fem export elem;xpote as test field reg $Box class $Grid iy 2
fem export elem;ypote as test field reg $Box class $Grid iy 3
fem export elem;zpote as test field reg $Box class $Grid iy 4

sub WriteGradPhiExelem {

  # This Perl subroutine is used to combine three potentential gradient
  # exelem files into a single three component field within a single
  # exelem file. This routine requires filenames for the three input
  # exelems to be passed and also a filename for the output exelem.

  ( $DXFILENAME, $DYFILENAME, $DZFILENAME, $OUTPUTNAME ) = @_;

  my $Found = 0;
  my $Found2 = 0;
  my $First = 1;
  my $InputLine = ' ';

  CORE::open( OUTPUTFILE,">$OUTPUTNAME" ) or die( "ERROR: Unable to open output file\n" );
  CORE::open( INPUTFILE,"<$DXFILENAME" ) or die( "ERROR: Unable to open DX input file\n" );

  while( <INPUTFILE> ) {
    if( /Nodes:/ ) {
      $Inputline = $_;
      $Found2 = 0;

      CORE::open( INPUTFILE2,"<$DYFILENAME" ) or die( "ERROR: Unable to open DY input file\n" );
      while( <INPUTFILE2> ) {
        if( /Nodes:/ ) {
          $Found2 = 0;
        }
        if( $Found2 ) {
          print( OUTPUTFILE "$_" );
        }
        if( /Values:/ ) {
          $Found2 = 1;
        }
      }
      CORE::close( INPUTFILE2 );

      $Found2 = 0;
      CORE::open( INPUTFILE2,"<$DZFILENAME" ) or die( "ERROR: Unable to open DZ input file\n" );
      while( <INPUTFILE2> ) {
        if( /Nodes:/ ) {
          $Found2 = 0;
        }
        if( $Found2 ) {
          print( OUTPUTFILE "$_" );
        }
        if( /Values:/ ) {
          $Found2 = 1;
        }
      }
      CORE::close( INPUTFILE2 );

      print( OUTPUTFILE "$Inputline" );
    }

    if( !$Found ) {
      print( OUTPUTFILE $_ );
    } elsif( $First ) {
      print( OUTPUTFILE ' 1) gradphi, field, rectangular cartesian, #Components=3'."\n");
      print( OUTPUTFILE '   x.  l.Lagrange*l.Lagrange*l.Lagrange, no modify, grid based.'."\n");
      $First = 0;
    } 
    if( /Fields/ ) {
      $Found = 1;
    }
    if( /xi1/ ) {
      print( OUTPUTFILE $_ );
      print( OUTPUTFILE '   y.  l.Lagrange*l.Lagrange*l.Lagrange, no modify, grid based.'."\n");
      print( OUTPUTFILE $_ );
      print( OUTPUTFILE '   z.  l.Lagrange*l.Lagrange*l.Lagrange, no modify, grid based.'."\n");
      print( OUTPUTFILE $_ );
    }
    if(( /Element/ ) && $Found ) {
      print( OUTPUTFILE $_ );
      $Found = 0;     
    }
  }
  CORE::close( INPUTFILE );
  CORE::close( OUTPUTFILE );
       
} # WriteGradPhiExelem

# Set up the filenames to pass to the Perl subroutine
$DXFILENAME = "xpote.exelem";
$DYFILENAME = "ypote.exelem";
$DZFILENAME = "zpote.exelem";
$OUTPUTNAME = "GradPhi.exelem";

# Call the Perl subroutine to generate the exelem file that can be read
# into CMGUI to create the streamlines.
&WriteGradPhiExelem( $DXFILENAME, $DYFILENAME, $DZFILENAME, $OUTPUTNAME );
		 
# ========================================================================

if( !$TESTING ) {
  # These commands illustrate how to draw current lines within a boundary
  # element torso. This relies on specifying seed points within the box enclosing the
  # torso. These seed points are xi positions. Streamlines are then calculated in the
  # forward and reverse directions from these seed points.

  # Read in the torso geometry (heart and torso BE regions)
  gfx read node hearttorso
  gfx read elem hearttorso

  # Read in the potential field defined by the boundary elements
  gfx read node field
  gfx read elem field

  # Read in the geometry of the box containing the torso
  gfx read node geom
  gfx read elem geom

  # Read in the potential gradient field. This needs to be a 3d (3 component)
  # vector field in order to draw streamlines. The components are dphi/dx,
  # dphi/dy, dphi/dz.
  gfx read elem GradPhi

  # Create a 3d viewing window
  gfx create window

  # Modify the default spectrum so it uses a colour range appropriate for our
  # potential values.
  gfx modify spectrum default linear reverse range -6.0 2.0 

  # Create a material called default+ which is the same as the default
  # material except that it has an alpha value of 0.3.
  gfx create material default+ alpha 0.3

  # These arrays represent the xi seed points in the box that encloses the torso. The
  # first array holds the seed points that create the streamlines below the level of the
  # heart and the second array holds the seed points that create the streamlines above
  # the heart.
  @SeedPointsLower = qw/ 0.5,0.5,0.5 0.42,0.5,0.5 0.35,0.5,0.5 0.33,0.5,0.5 0.31,0.5,0.5 0.295,0.5,0.5 0.28,0.5,0.5 /;
  @SeedPointsUpper = qw/ 0.5,0.5,0.7 0.5,0.5,0.75 0.5,0.5,0.8 0.5,0.5,0.85 0.5,0.5,0.9 /;

  # Concatenate the seed point arrays into a single array
  @SeedPoints = ( @SeedPointsLower, @SeedPointsUpper );

  # We deal with reasonably small potentials and that means even smaller potential
  # gradients. The streamline tracking code takes step sizes based on vector lengths
  # so here the vector gradient field is scaled up 1000 times. This is equivalent to
  # a linear scaling of the potential field and therefore does not affect the 
  # streamlines. 
  gfx define field ScalGradPhi scale field gradphi scale 1000 1000 1000

  # Loop over each of the seed points in turn and create streamlines that have
  # a rectangular cross section in both the forward and reverse directions from
  # each seed point. These streamlines can be drawn much faster using lines instead
  # of rectangles, or much slower using ellipses. The quality of the streamlines
  # depends on the resolution of the grid points that contain the gradient information.
  foreach my $seed ( 1..@SeedPoints ) {
    gfx modify g_element test streamlines xi $SeedPoints[$seed-1] rectangle vector ScalGradPhi length 1000 width 10 material default_selected
    gfx modify g_element test streamlines xi $SeedPoints[$seed-1] rectangle vector ScalGradPhi reverse_track length 1000 width 15 material default_selected
  }

  # Create an element group that contains the epicardial surface elements and draw
  # surfaces over the epicardium that are coloured by the potential field.
  gfx create egroup heart add 1..40
  gfx modify g_element heart surfaces material default data potential spectrum default

  # Create an element group that contains the body surface elements and draw
  # surfaces over the torso that are coloured by the potential field. Note that
  # this uses the transparent material so the streamlines are visible inside.
  gfx create egroup torso add 41..344
  gfx modify g_element torso surfaces material default+ data potential spectrum default

  # Use slow transparency in the graphics window so the torso surfaces streamlines
  # are drawn in the correct order.
  gfx modify window 1 set slow_trans

  gfx
}

